@@ -1,5 +1,5 @@
 /*
-Copyright 2022-2024 MetaOPT Team. All Rights Reserved.
+Copyright 2022-2025 MetaOPT Team. All Rights Reserved.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -17,7 +17,6 @@ limitations under the License.
 
 #pragma once
 
-#include <algorithm>      // std::min
 #include <memory>         // std::unique_ptr
 #include <optional>       // std::optional, std::nullopt
 #include <string>         // std::string
@@ -41,13 +40,12 @@ using size_t = py::size_t;
 using ssize_t = py::ssize_t;
 
 // The maximum depth of a pytree.
-#ifndef Py_C_RECURSION_LIMIT
-#define Py_C_RECURSION_LIMIT 1000
-#endif
-#if !defined(PYPY_VERSION) && !(defined(MS_WINDOWS) && defined(Py_DEBUG))
-constexpr ssize_t MAX_RECURSION_DEPTH = std::min(1000, Py_C_RECURSION_LIMIT);
+#if defined(Py_DEBUG) || defined(PYPY_VERSION) || defined(MS_WINDOWS) ||                           \
+    (defined(__wasm__) || defined(__wasm32__) || defined(__wasm64__) || defined(__wasi__) ||       \
+     defined(__EMSCRIPTEN__))
+constexpr ssize_t MAX_RECURSION_DEPTH = 500;
 #else
-constexpr ssize_t MAX_RECURSION_DEPTH = std::min(500, Py_C_RECURSION_LIMIT);
+constexpr ssize_t MAX_RECURSION_DEPTH = 1000;
 #endif
 
 // Test whether the given object is a leaf node.
@@ -119,10 +117,11 @@ public:
     // Return an unflattened PyTree given an iterable of leaves and a PyTreeSpec.
     [[nodiscard]] py::object Unflatten(const py::iterable &leaves) const;
 
-    // Flatten a PyTree up to this PyTreeSpec. 'this' must be a tree prefix of the tree-structure
-    // of 'x'. For example, if we flatten a value [(1, (2, 3)), {"foo": 4}] with a PyTreeSpec [(*,
-    // *), *], the result is the list of leaves [1, (2, 3), {"foo": 4}].
-    [[nodiscard]] py::list FlattenUpTo(const py::object &full_tree) const;
+    // Flatten a PyTree up to this PyTreeSpec. 'this' must be a tree prefix of the tree-structure of
+    // 'tree'.
+    // For example, if we flatten a value [(1, (2, 3)), {"foo": 4}] with a PyTreeSpec([(*, *), *]),
+    // the result is the list of leaves [1, (2, 3), {"foo": 4}].
+    [[nodiscard]] py::list FlattenUpTo(const py::object &tree) const;
 
     // Broadcast to a common suffix of this PyTreeSpec and other PyTreeSpec.
     [[nodiscard]] std::unique_ptr<PyTreeSpec> BroadcastToCommonSuffix(
@@ -135,10 +134,17 @@ public:
         const std::optional<py::function> &f_leaf = std::nullopt) const;
 
     // Compose two PyTreeSpecs, replacing the leaves of this tree with copies of `inner`.
-    [[nodiscard]] std::unique_ptr<PyTreeSpec> Compose(const PyTreeSpec &inner_treespec) const;
+    [[nodiscard]] std::unique_ptr<PyTreeSpec> Compose(const PyTreeSpec &inner) const;
 
-    // Map a function over a PyTree structure, applying f_leaf to each leaf, and
-    // f_node(node_type, node_data, children) to each container node.
+    // Map a function over a PyTree structure, applying `f_leaf(leaf)` to each leaf,
+    // and `f_node(node)` to each reconstructed non-leaf node.
+    [[nodiscard]] py::object Traverse(
+        const py::iterable &leaves,
+        const std::optional<py::function> &f_node = std::nullopt,
+        const std::optional<py::function> &f_leaf = std::nullopt) const;
+
+    // Map a function over a PyTree structure, applying `f_leaf(leaf)` to each leaf,
+    // and `f_node(node_type, node_data, children)` to each non-leaf node.
     [[nodiscard]] py::object Walk(const py::iterable &leaves,
                                   const std::optional<py::function> &f_node = std::nullopt,
                                   const std::optional<py::function> &f_leaf = std::nullopt) const;
@@ -265,7 +271,7 @@ public:
     static inline Py_ALWAYS_INLINE bool IsDictInsertionOrdered(
         const std::string &registry_namespace,
         const bool &inherit_global_namespace = true) {
-        const scoped_read_lock_guard lock{sm_is_dict_insertion_ordered_mutex};
+        const scoped_read_lock lock{sm_is_dict_insertion_ordered_mutex};
 
         return (sm_is_dict_insertion_ordered.find(registry_namespace) !=
                 sm_is_dict_insertion_ordered.end()) ||
@@ -277,7 +283,7 @@ public:
     static inline Py_ALWAYS_INLINE void SetDictInsertionOrdered(
         const bool &mode,
         const std::string &registry_namespace) {
-        const scoped_write_lock_guard lock{sm_is_dict_insertion_ordered_mutex};
+        const scoped_write_lock lock{sm_is_dict_insertion_ordered_mutex};
 
         if (mode) [[likely]] {
             sm_is_dict_insertion_ordered.insert(registry_namespace);
@@ -353,9 +359,9 @@ private:
                      const bool &none_is_leaf,
                      const std::string &registry_namespace);
 
-    template <bool NoneIsLeaf, bool DictShouldBeSorted, typename Span>
+    template <bool NoneIsLeaf, bool DictShouldBeSorted, typename Vector>
     bool FlattenIntoImpl(const py::handle &handle,
-                         Span &leaves,  // NOLINT[runtime/references]
+                         Vector &leaves,  // NOLINT[runtime/references]
                          const ssize_t &depth,
                          const std::optional<py::function> &leaf_predicate,
                          const std::string &registry_namespace);
@@ -370,13 +376,13 @@ private:
 
     template <bool NoneIsLeaf,
               bool DictShouldBeSorted,
-              typename LeafSpan,
-              typename PathSpan,
+              typename LeafVector,
+              typename PathVector,
               typename Stack>
     bool FlattenIntoWithPathImpl(const py::handle &handle,
-                                 LeafSpan &leaves,  // NOLINT[runtime/references]
-                                 PathSpan &paths,   // NOLINT[runtime/references]
-                                 Stack &stack,      // NOLINT[runtime/references]
+                                 LeafVector &leaves,  // NOLINT[runtime/references]
+                                 PathVector &paths,   // NOLINT[runtime/references]
+                                 Stack &stack,        // NOLINT[runtime/references]
                                  const ssize_t &depth,
                                  const std::optional<py::function> &leaf_predicate,
                                  const std::string &registry_namespace);
@@ -391,15 +397,21 @@ private:
         const std::vector<Node> &other_traversal,
         const ssize_t &other_pos);
 
-    template <typename Span, typename Stack>
-    [[nodiscard]] ssize_t PathsImpl(Span &paths,   // NOLINT[runtime/references]
-                                    Stack &stack,  // NOLINT[runtime/references]
+    template <bool PassRawNode = true>
+    [[nodiscard]] py::object WalkImpl(
+        const py::iterable &leaves,
+        const std::optional<py::function> &f_node = std::nullopt,
+        const std::optional<py::function> &f_leaf = std::nullopt) const;
+
+    template <typename PathVector, typename Stack>
+    [[nodiscard]] ssize_t PathsImpl(PathVector &paths,  // NOLINT[runtime/references]
+                                    Stack &stack,       // NOLINT[runtime/references]
                                     const ssize_t &pos,
                                     const ssize_t &depth) const;
 
-    template <typename Span, typename Stack>
-    [[nodiscard]] ssize_t AccessorsImpl(Span &accessors,  // NOLINT[runtime/references]
-                                        Stack &stack,     // NOLINT[runtime/references]
+    template <typename AccessorVector, typename Stack>
+    [[nodiscard]] ssize_t AccessorsImpl(AccessorVector &accessors,  // NOLINT[runtime/references]
+                                        Stack &stack,               // NOLINT[runtime/references]
                                         const ssize_t &pos,
                                         const ssize_t &depth) const;
 
@@ -414,6 +426,9 @@ private:
     // Used in tp_traverse for GC support.
     static int PyTpTraverse(PyObject *self_base, visitproc visit, void *arg);
 
+    // Used in tp_clear for GC support.
+    static int PyTpClear(PyObject *self_base);
+
     // A set of namespaces that preserve the insertion order of the dictionary keys during
     // flattening.
     static inline std::unordered_set<std::string> sm_is_dict_insertion_ordered{};
@@ -448,13 +463,13 @@ public:
     friend void BuildModule(py::module_ &mod);  // NOLINT[runtime/references]
 
 private:
-    const py::object m_root;
+    py::object m_root;
     std::vector<std::pair<py::object, ssize_t>> m_agenda;
     const std::optional<py::function> m_leaf_predicate;
     const bool m_none_is_leaf;
     const std::string m_namespace;
     const bool m_is_dict_insertion_ordered;
-#ifdef Py_GIL_DISABLED
+#if defined(Py_GIL_DISABLED)
     mutable mutex m_mutex{};
 #endif
 
@@ -463,6 +478,9 @@ private:
 
     // Used in tp_traverse for GC support.
     static int PyTpTraverse(PyObject *self_base, visitproc visit, void *arg);
+
+    // Used in tp_clear for GC support.
+    static int PyTpClear(PyObject *self_base);
 };
 
 }  // namespace optree
