@@ -1,4 +1,4 @@
-# Copyright 2022-2024 MetaOPT Team. All Rights Reserved.
+# Copyright 2022-2025 MetaOPT Team. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,8 +16,8 @@
 
 # pylint: disable=invalid-name
 
-__version__ = '0.14.0'
-__license__ = 'Apache License, Version 2.0'
+__version__ = '0.18.0'
+__license__ = 'Apache-2.0'
 __author__ = 'OpTree Contributors'
 __release__ = True
 
@@ -25,13 +25,20 @@ if not __release__:
     import os
     import subprocess
 
+    root_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
     try:
         prefix, sep, suffix = (
             subprocess.check_output(  # noqa: S603
-                ['git', 'describe', '--abbrev=7'],  # noqa: S607
-                cwd=os.path.dirname(os.path.abspath(__file__)),
+                [  # noqa: S607
+                    'git',
+                    f'--git-dir={os.path.join(root_dir, ".git")}',
+                    'describe',
+                    '--abbrev=7',
+                ],
+                cwd=root_dir,
                 stderr=subprocess.DEVNULL,
                 text=True,
+                encoding='utf-8',
             )
             .strip()
             .lstrip('v')
@@ -50,4 +57,4 @@ if not __release__:
     except (OSError, subprocess.CalledProcessError):
         pass
 
-    del os, subprocess
+    del os, subprocess, root_dir
