@@ -1,5 +1,5 @@
 /*
-Copyright 2022-2024 MetaOPT Team. All Rights Reserved.
+Copyright 2022-2025 MetaOPT Team. All Rights Reserved.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -23,11 +23,11 @@ limitations under the License.
 namespace optree {
 
 template <typename Span>
-py::object PyTreeSpec::UnflattenImpl(const Span& leaves) const {
+py::object PyTreeSpec::UnflattenImpl(const Span &leaves) const {
     auto agenda = reserved_vector<py::object>(4);
     auto it = leaves.begin();
     ssize_t num_leaves = 0;
-    for (const Node& node : m_traversal) {
+    for (const Node &node : m_traversal) {
         EXPECT_GE(py::ssize_t_cast(agenda.size()),
                   node.arity,
                   "Too few elements for PyTreeSpec node.");
@@ -58,13 +58,14 @@ py::object PyTreeSpec::UnflattenImpl(const Span& leaves) const {
             case PyTreeKind::Custom: {
                 const ssize_t size = py::ssize_t_cast(agenda.size());
                 py::object out = MakeNode(node,
-                                          (node.arity > 0 ? &agenda[size - node.arity] : nullptr),
+                                          node.arity > 0 ? &agenda[size - node.arity] : nullptr,
                                           node.arity);
                 agenda.resize(size - node.arity);
                 agenda.emplace_back(std::move(out));
                 break;
             }
 
+            case PyTreeKind::NumKinds:
             default:
                 INTERNAL_ERROR();
         }
@@ -78,7 +79,7 @@ py::object PyTreeSpec::UnflattenImpl(const Span& leaves) const {
     return agenda.back();
 }
 
-py::object PyTreeSpec::Unflatten(const py::iterable& leaves) const {
+py::object PyTreeSpec::Unflatten(const py::iterable &leaves) const {
     PYTREESPEC_SANITY_CHECK(*this);
 
     const scoped_critical_section cs{leaves};
