@@ -1,4 +1,4 @@
-# Copyright 2022-2024 MetaOPT Team. All Rights Reserved.
+# Copyright 2022-2025 MetaOPT Team. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,45 +16,54 @@
 # pylint: disable=missing-function-docstring,invalid-name
 
 import re
+import sys
 import weakref
 from collections import UserDict, UserList, namedtuple
+from dataclasses import dataclass
 
 import pytest
 
 import optree
 import optree._C
-from helpers import GLOBAL_NAMESPACE, Py_GIL_DISABLED, gc_collect, skipif_pypy
+from helpers import (
+    GLOBAL_NAMESPACE,
+    PYPY,
+    Py_GIL_DISABLED,
+    disable_systrace,
+    gc_collect,
+    skipif_pypy,
+)
 
 
 def test_register_pytree_node_class_with_no_namespace():
     with pytest.raises(
         ValueError,
-        match='Must specify `namespace` when the first argument is a class.',
+        match=re.escape('Must specify `namespace` when the first argument is a class.'),
     ):
 
         @optree.register_pytree_node_class
         class MyList(UserList):
-            def tree_flatten(self):
+            def __tree_flatten__(self):
                 return self.data, None, None
 
             @classmethod
-            def tree_unflatten(cls, metadata, children):
+            def __tree_unflatten__(cls, metadata, children):
                 return cls(children)
 
 
 def test_register_pytree_node_class_with_duplicate_namespace():
     with pytest.raises(
         ValueError,
-        match='Cannot specify `namespace` when the first argument is a string.',
+        match=re.escape('Cannot specify `namespace` when the first argument is a string.'),
     ):
 
         @optree.register_pytree_node_class('mylist', namespace='mylist')
         class MyList(UserList):
-            def tree_flatten(self):
+            def __tree_flatten__(self):
                 return self.data, None, None
 
             @classmethod
-            def tree_unflatten(cls, metadata, children):
+            def __tree_unflatten__(cls, metadata, children):
                 return cls(children)
 
 
@@ -91,20 +100,20 @@ def test_register_pytree_node_with_non_class():
 def test_register_pytree_node_class_with_duplicate_registrations():
     @optree.register_pytree_node_class('mylist1')
     class MyList1(UserList):
-        def tree_flatten(self):
+        def __tree_flatten__(self):
             return self.data, None, None
 
         @classmethod
-        def tree_unflatten(cls, metadata, children):
+        def __tree_unflatten__(cls, metadata, children):
             return cls(children)
 
     @optree.register_pytree_node_class(namespace='mylist2')
     class MyList2(UserList):
-        def tree_flatten(self):
+        def __tree_flatten__(self):
             return self.data, None, None
 
         @classmethod
-        def tree_unflatten(cls, metadata, children):
+        def __tree_unflatten__(cls, metadata, children):
             return cls(children)
 
     with pytest.raises(
@@ -126,33 +135,33 @@ def test_register_pytree_node_with_invalid_namespace():
 
         @optree.register_pytree_node_class(namespace=1)
         class MyList1(UserList):
-            def tree_flatten(self):
+            def __tree_flatten__(self):
                 return self.data, None, None
 
             @classmethod
-            def tree_unflatten(cls, metadata, children):
+            def __tree_unflatten__(cls, metadata, children):
                 return cls(children)
 
-    with pytest.raises(ValueError, match='The namespace cannot be an empty string.'):
+    with pytest.raises(ValueError, match=re.escape('The namespace cannot be an empty string.')):
 
         @optree.register_pytree_node_class('')
         class MyList2(UserList):
-            def tree_flatten(self):
+            def __tree_flatten__(self):
                 return self.data, None, None
 
             @classmethod
-            def tree_unflatten(cls, metadata, children):
+            def __tree_unflatten__(cls, metadata, children):
                 return cls(children)
 
-    with pytest.raises(ValueError, match='The namespace cannot be an empty string.'):
+    with pytest.raises(ValueError, match=re.escape('The namespace cannot be an empty string.')):
 
         @optree.register_pytree_node_class(namespace='')
         class MyList3(UserList):
-            def tree_flatten(self):
+            def __tree_flatten__(self):
                 return self.data, None, None
 
             @classmethod
-            def tree_unflatten(cls, metadata, children):
+            def __tree_unflatten__(cls, metadata, children):
                 return cls(children)
 
     with pytest.raises(TypeError, match='The namespace must be a string'):
@@ -163,7 +172,7 @@ def test_register_pytree_node_with_invalid_namespace():
             namespace=1,
         )
 
-    with pytest.raises(ValueError, match='The namespace cannot be an empty string.'):
+    with pytest.raises(ValueError, match=re.escape('The namespace cannot be an empty string.')):
         optree.register_pytree_node(
             set,
             lambda s: (sorted(s), None, None),
@@ -179,11 +188,11 @@ def test_register_pytree_node_with_invalid_path_entry_type():
         class MyList1(UserList):
             TREE_PATH_ENTRY_TYPE = None
 
-            def tree_flatten(self):
+            def __tree_flatten__(self):
                 return self.data, None, None
 
             @classmethod
-            def tree_unflatten(cls, metadata, children):
+            def __tree_unflatten__(cls, metadata, children):
                 return cls(children)
 
     with pytest.raises(TypeError, match=r'Expected a subclass of PyTreeEntry, got .*\.'):
@@ -192,33 +201,33 @@ def test_register_pytree_node_with_invalid_path_entry_type():
         class MyList2(UserList):
             TREE_PATH_ENTRY_TYPE = int
 
-            def tree_flatten(self):
+            def __tree_flatten__(self):
                 return self.data, None, None
 
             @classmethod
-            def tree_unflatten(cls, metadata, children):
+            def __tree_unflatten__(cls, metadata, children):
                 return cls(children)
 
     with pytest.raises(TypeError, match=r'Expected a subclass of PyTreeEntry, got .*\.'):
 
         @optree.register_pytree_node_class(path_entry_type=1, namespace='error')
         class MyList3(UserList):
-            def tree_flatten(self):
+            def __tree_flatten__(self):
                 return self.data, None, None
 
             @classmethod
-            def tree_unflatten(cls, metadata, children):
+            def __tree_unflatten__(cls, metadata, children):
                 return cls(children)
 
     with pytest.raises(TypeError, match=r'Expected a subclass of PyTreeEntry, got .*\.'):
 
         @optree.register_pytree_node_class(path_entry_type=int, namespace='error')
         class MyList4(UserList):
-            def tree_flatten(self):
+            def __tree_flatten__(self):
                 return self.data, None, None
 
             @classmethod
-            def tree_unflatten(cls, metadata, children):
+            def __tree_unflatten__(cls, metadata, children):
                 return cls(children)
 
     with pytest.raises(TypeError, match=r'Expected a subclass of PyTreeEntry, got .*\.'):
@@ -390,11 +399,11 @@ def test_register_pytree_node_namedtuple():
 def test_flatten_with_wrong_number_of_returns():
     @optree.register_pytree_node_class(namespace='error')
     class MyList1(UserList):
-        def tree_flatten(self):
+        def __tree_flatten__(self):
             return (self.data,)
 
         @classmethod
-        def tree_unflatten(cls, metadata, children):
+        def __tree_unflatten__(cls, metadata, children):
             return cls(children)
 
     with pytest.raises(
@@ -411,11 +420,11 @@ def test_flatten_with_wrong_number_of_returns():
 
     @optree.register_pytree_node_class(namespace='error')
     class MyList4(UserList):
-        def tree_flatten(self):
+        def __tree_flatten__(self):
             return self.data, None, None, None
 
         @classmethod
-        def tree_unflatten(cls, metadata, children):
+        def __tree_unflatten__(cls, metadata, children):
             return cls(children)
 
     with pytest.raises(
@@ -432,11 +441,11 @@ def test_flatten_with_wrong_number_of_returns():
 
     @optree.register_pytree_node_class(namespace='error')
     class MyListEntryMismatch(UserList):
-        def tree_flatten(self):
+        def __tree_flatten__(self):
             return self.data, None, range(len(self) + 1)
 
         @classmethod
-        def tree_unflatten(cls, metadata, children):
+        def __tree_unflatten__(cls, metadata, children):
             return cls(children)
 
     with pytest.raises(
@@ -453,18 +462,68 @@ def test_flatten_with_wrong_number_of_returns():
 
 
 def test_pytree_node_registry_get():
+    registry1 = optree.register_pytree_node.get()
+    assert optree.register_pytree_node.get() == registry1
+    assert optree.register_pytree_node.get(namespace=GLOBAL_NAMESPACE) == registry1
+    assert optree.register_pytree_node.get(namespace='undefined') == registry1
+    assert optree.register_pytree_node.get() is not registry1
+    assert all(handler.namespace == '' for handler in registry1.values())
+    with optree.dict_insertion_ordered(True, namespace=GLOBAL_NAMESPACE):
+        registry2 = optree.register_pytree_node.get()
+        assert registry2 != registry1
+        assert list(registry2) == list(registry1)
+        assert optree.register_pytree_node.get() == registry2
+        assert optree.register_pytree_node.get(namespace='undefined') == registry2
+        assert optree.register_pytree_node.get() is not registry2
+        assert all(handler.namespace == '' for handler in registry2.values())
+
     handler = optree.register_pytree_node.get(list)
     assert handler is not None
+    assert handler is registry1[list]
+    assert handler is registry2[list]
     lst = [1, 2, 3]
     assert tuple(handler.flatten_func(lst))[:2] == (lst, None)
 
     handler = optree.register_pytree_node.get(list, namespace='any')
     assert handler is not None
+    assert handler is registry1[list]
+    assert handler is registry2[list]
     lst = [1, 2, 3]
     assert tuple(handler.flatten_func(lst))[:2] == (lst, None)
 
+    handler = optree.register_pytree_node.get(dict)
+    assert handler is not None
+    assert handler is registry1[dict]
+    assert handler is not registry2[dict]
+    dct = {'b': 2, 'c': 3, 'a': 1}
+    assert tuple(handler.flatten_func(dct))[:2] == ((1, 2, 3), ['a', 'b', 'c'])
+
+    handler = optree.register_pytree_node.get(dict, namespace='any')
+    assert handler is not None
+    assert handler is registry1[dict]
+    assert handler is not registry2[dict]
+    dct = {'b': 2, 'c': 3, 'a': 1}
+    assert tuple(handler.flatten_func(dct))[:2] == ((1, 2, 3), ['a', 'b', 'c'])
+
+    with optree.dict_insertion_ordered(True, namespace=GLOBAL_NAMESPACE):
+        handler = optree.register_pytree_node.get(dict)
+        assert handler is not None
+        assert handler is registry2[dict]
+        assert handler is not registry1[dict]
+        dct = {'b': 2, 'c': 3, 'a': 1}
+        assert tuple(handler.flatten_func(dct))[:2] == ((2, 3, 1), ['b', 'c', 'a'])
+
+        handler = optree.register_pytree_node.get(dict, namespace='any')
+        assert handler is not None
+        assert handler is registry2[dict]
+        assert handler is not registry1[dict]
+        dct = {'b': 2, 'c': 3, 'a': 1}
+        assert tuple(handler.flatten_func(dct))[:2] == ((2, 3, 1), ['b', 'c', 'a'])
+
     handler = optree.register_pytree_node.get(set)
     assert handler is None
+    assert set not in registry1
+    assert set not in registry2
 
     optree.register_pytree_node(
         set,
@@ -472,27 +531,59 @@ def test_pytree_node_registry_get():
         lambda _, s: set(s),
         namespace=GLOBAL_NAMESPACE,
     )
+    registry3 = optree.register_pytree_node.get()
     handler = optree.register_pytree_node.get(set)
     assert handler is not None
+    assert set in registry3
+    assert handler is registry3[set]
+    assert list(registry1) == list(registry3)[:-1]
 
     handler = optree.register_pytree_node.get(set, namespace='set')
     assert handler is not None
+    assert optree.register_pytree_node.get(namespace='set') == registry3
 
     @optree.register_pytree_node_class(namespace='mylist')
     class MyList(UserList):
-        def tree_flatten(self):
+        def __tree_flatten__(self):
             return self.data, None, None
 
         @classmethod
-        def tree_unflatten(cls, metadata, children):
+        def __tree_unflatten__(cls, metadata, children):
             return cls(children)
 
     handler = optree.register_pytree_node.get(MyList)
     assert handler is None
+    assert optree.register_pytree_node.get() == registry3
+    assert MyList not in registry3
     handler = optree.register_pytree_node.get(MyList, namespace='set')
     assert handler is None
+    assert MyList not in registry3
+
     handler = optree.register_pytree_node.get(MyList, namespace='mylist')
+    registry4 = optree.register_pytree_node.get(namespace='mylist')
     assert handler is not None
+    assert MyList in registry4
+    assert handler is registry4[MyList]
+    assert list(registry3) == list(registry4)[: len(registry3)]
+    for cls, handler in registry4.items():
+        assert handler.type is cls
+        if cls not in registry3:
+            assert handler.namespace == 'mylist'
+        else:
+            assert handler.namespace == ''
+            assert handler is registry3[cls]
+
+
+def test_pytree_node_registry_get_with_invalid_arguments():
+    registry = optree.register_pytree_node.get()
+    assert optree.register_pytree_node.get(None) == registry
+    assert optree.register_pytree_node.get(namespace=GLOBAL_NAMESPACE) == registry
+    assert optree.register_pytree_node.get(namedtuple) is registry[namedtuple]  # noqa: PYI024
+    if sys.version_info[:2] != (3, 9) or PYPY:
+        with pytest.raises(TypeError, match='Expected a class or None'):
+            optree.register_pytree_node.get(dataclass)
+        with pytest.raises(TypeError, match='The namespace must be a string'):
+            optree.register_pytree_node.get(list, namespace=None)
 
 
 def test_pytree_node_registry_with_init_subclass():
@@ -502,12 +593,12 @@ def test_pytree_node_registry_with_init_subclass():
             super().__init_subclass__()
             optree.register_pytree_node_class(cls, namespace='mydict')
 
-        def tree_flatten(self):
+        def __tree_flatten__(self):
             reversed_keys = sorted(self.keys(), reverse=True)
             return [self[key] for key in reversed_keys], reversed_keys, reversed_keys
 
         @classmethod
-        def tree_unflatten(cls, metadata, children):
+        def __tree_unflatten__(cls, metadata, children):
             return cls(zip(metadata, children))
 
     class MyAnotherDict(MyDict):
@@ -565,6 +656,9 @@ def test_pytree_node_registry_with_init_subclass():
 
 
 def test_unregister_pytree_node_with_non_class():
+    with pytest.raises(TypeError, match='Expected a class'):
+        optree.unregister_pytree_node(dataclass, namespace=GLOBAL_NAMESPACE)
+
     def func1():
         pass
 
@@ -586,11 +680,11 @@ def test_unregister_pytree_node_with_non_class():
 
 def test_unregister_pytree_node_with_non_registered_class():
     class MyList(UserList):
-        def tree_flatten(self):
+        def __tree_flatten__(self):
             return self.data, None, None
 
         @classmethod
-        def tree_unflatten(cls, metadata, children):
+        def __tree_unflatten__(cls, metadata, children):
             return cls(children)
 
     with pytest.raises(
@@ -626,7 +720,7 @@ def test_unregister_pytree_node_with_invalid_namespace():
     with pytest.raises(TypeError, match='The namespace must be a string'):
         optree.unregister_pytree_node(set, namespace=1)
 
-    with pytest.raises(ValueError, match='The namespace cannot be an empty string.'):
+    with pytest.raises(ValueError, match=re.escape('The namespace cannot be an empty string.')):
         optree.unregister_pytree_node(set, namespace='')
 
 
@@ -767,14 +861,15 @@ def test_unregister_pytree_node_namedtuple():
 
 
 @skipif_pypy
+@disable_systrace
 def test_unregister_pytree_node_no_reference_leak():  # noqa: C901
     @optree.register_pytree_node_class(namespace=GLOBAL_NAMESPACE)
     class MyList1(UserList):
-        def tree_flatten(self):
+        def __tree_flatten__(self):
             return self.data, None, None
 
         @classmethod
-        def tree_unflatten(cls, metadata, children):
+        def __tree_unflatten__(cls, metadata, children):
             return cls(children)
 
     wr = weakref.ref(MyList1)
@@ -788,11 +883,11 @@ def test_unregister_pytree_node_no_reference_leak():  # noqa: C901
 
     @optree.register_pytree_node_class(namespace=GLOBAL_NAMESPACE)
     class MyList2(UserList):
-        def tree_flatten(self):
+        def __tree_flatten__(self):
             return reversed(self.data), None, None
 
         @classmethod
-        def tree_unflatten(cls, metadata, children):
+        def __tree_unflatten__(cls, metadata, children):
             return cls(reversed(children))
 
     wr = weakref.ref(MyList2)
@@ -816,11 +911,11 @@ def test_unregister_pytree_node_no_reference_leak():  # noqa: C901
 
     @optree.register_pytree_node_class(namespace=GLOBAL_NAMESPACE)
     class MyList3(UserList):
-        def tree_flatten(self):
+        def __tree_flatten__(self):
             return reversed(self.data), None, None
 
         @classmethod
-        def tree_unflatten(cls, metadata, children):
+        def __tree_unflatten__(cls, metadata, children):
             return cls(reversed(children))
 
     wr = weakref.ref(MyList3)
@@ -847,11 +942,11 @@ def test_unregister_pytree_node_no_reference_leak():  # noqa: C901
 
     @optree.register_pytree_node_class(namespace='mylist')
     class MyList4(UserList):
-        def tree_flatten(self):
+        def __tree_flatten__(self):
             return self.data, None, None
 
         @classmethod
-        def tree_unflatten(cls, metadata, children):
+        def __tree_unflatten__(cls, metadata, children):
             return cls(children)
 
     wr = weakref.ref(MyList4)
@@ -865,11 +960,11 @@ def test_unregister_pytree_node_no_reference_leak():  # noqa: C901
 
     @optree.register_pytree_node_class(namespace='mylist')
     class MyList5(UserList):
-        def tree_flatten(self):
+        def __tree_flatten__(self):
             return reversed(self.data), None, None
 
         @classmethod
-        def tree_unflatten(cls, metadata, children):
+        def __tree_unflatten__(cls, metadata, children):
             return cls(reversed(children))
 
     wr = weakref.ref(MyList5)
@@ -898,7 +993,7 @@ def test_dict_insertion_order_with_invalid_namespace():
     with pytest.raises(TypeError, match='The namespace must be a string'):
         with optree.dict_insertion_ordered(True, namespace=1):
             pass
-    with pytest.raises(ValueError, match='The namespace cannot be an empty string.'):
+    with pytest.raises(ValueError, match=re.escape('The namespace cannot be an empty string.')):
         with optree.dict_insertion_ordered(True, namespace=''):
             pass
 
@@ -1023,3 +1118,158 @@ def test_dict_insertion_order_with_nested_context():
     assert not is_dict_insertion_ordered('')
     assert not is_dict_insertion_ordered('namespace')
     assert not is_dict_insertion_ordered('other-namespace')
+
+
+def test_register_pytree_node_class_legacy_methods():
+    @optree.register_pytree_node_class(namespace='legacy')
+    class LegacyStyleClass(UserList):
+        def tree_flatten(self):
+            return self.data, None, None
+
+        @classmethod
+        def tree_unflatten(cls, metadata, children):
+            return cls(children)
+
+    assert hasattr(LegacyStyleClass, '__tree_flatten__')
+    assert hasattr(LegacyStyleClass, '__tree_unflatten__')
+    assert callable(LegacyStyleClass.__tree_flatten__)
+    assert callable(LegacyStyleClass.__tree_unflatten__)
+
+    tree = LegacyStyleClass([1, 2, 3])
+    leaves, treespec = optree.tree_flatten(tree, namespace='legacy')
+    assert leaves == [1, 2, 3]
+    assert tree == optree.tree_unflatten(treespec, leaves)
+
+    @optree.register_pytree_node_class(namespace='mixed')
+    class MixedStyleClass(UserList):
+        def __tree_flatten__(self):
+            return self.data, None, None
+
+        @classmethod
+        def __tree_unflatten__(cls, metadata, children):
+            return cls(children)
+
+        def tree_flatten(self):
+            return reversed(self.data), None, None
+
+        @classmethod
+        def tree_unflatten(cls, metadata, children):
+            return cls(reversed(children))
+
+    tree = MixedStyleClass([1, 2, 3])
+    leaves, treespec = optree.tree_flatten(tree, namespace='mixed')
+    assert leaves == [1, 2, 3]
+    assert tree == optree.tree_unflatten(treespec, leaves)
+
+
+def test_register_pytree_node_class_missing_methods():
+    with pytest.raises(
+        TypeError,
+        match=r'must define both `__tree_flatten__` and `__tree_unflatten__` methods',
+    ):
+
+        @optree.register_pytree_node_class(namespace='error')
+        class NoMethodsClass(UserList):
+            pass
+
+    with pytest.raises(
+        TypeError,
+        match=r'must define both `__tree_flatten__` and `__tree_unflatten__` methods',
+    ):
+
+        @optree.register_pytree_node_class(namespace='error')
+        class OnlyOldFlattenClass(UserList):
+            def tree_flatten(self):
+                return self.data, None, None
+
+    with pytest.raises(
+        TypeError,
+        match=r'must define both `__tree_flatten__` and `__tree_unflatten__` methods',
+    ):
+
+        @optree.register_pytree_node_class(namespace='error')
+        class OnlyOldUnflattenClass(UserList):
+            @classmethod
+            def tree_unflatten(cls, metadata, children):
+                return cls(children)
+
+    with pytest.raises(
+        TypeError,
+        match=r'must define both `__tree_flatten__` and `__tree_unflatten__` methods',
+    ):
+
+        @optree.register_pytree_node_class(namespace='error')
+        class OnlyNewFlattenClass(UserList):
+            def __tree_flatten__(self):
+                return self.data, None, None
+
+    with pytest.raises(
+        TypeError,
+        match=r'must define both `__tree_flatten__` and `__tree_unflatten__` methods',
+    ):
+
+        @optree.register_pytree_node_class(namespace='error')
+        class OnlyNewUnflattenClass(UserList):
+            @classmethod
+            def __tree_unflatten__(cls, metadata, children):
+                return cls(children)
+
+    with pytest.raises(
+        TypeError,
+        match=r'must define both `__tree_flatten__` and `__tree_unflatten__` methods',
+    ):
+
+        @optree.register_pytree_node_class(namespace='error')
+        class MixedIncompleteClass1(UserList):
+            def tree_flatten(self):
+                return self.data, None, None
+
+            @classmethod
+            def __tree_unflatten__(cls, metadata, children):
+                return cls(children)
+
+    with pytest.raises(
+        TypeError,
+        match=r'must define both `__tree_flatten__` and `__tree_unflatten__` methods',
+    ):
+
+        @optree.register_pytree_node_class(namespace='error')
+        class MixedIncompleteClass2(UserList):
+            def __tree_flatten__(self):
+                return self.data, None, None
+
+            @classmethod
+            def tree_unflatten(cls, metadata, children):
+                return cls(children)
+
+
+def test_register_pytree_node_class_wrapping_behavior():
+    @optree.register_pytree_node_class(namespace='wrapping')
+    class WrappingTestClass(UserList):
+        def tree_flatten(self):
+            return list(reversed(self.data)), 'reversed', None
+
+        @classmethod
+        def tree_unflatten(cls, metadata, children):
+            if metadata == 'reversed':
+                return cls(reversed(children))
+            return cls(children)
+
+    tree = WrappingTestClass([1, 2, 3])
+
+    children, metadata, entries = tree.__tree_flatten__()
+    assert list(children) == [3, 2, 1]
+    assert metadata == 'reversed'
+    assert entries is None
+
+    reconstructed = WrappingTestClass.__tree_unflatten__(metadata, children)
+    assert list(reconstructed) == [1, 2, 3]
+
+    leaves, treespec = optree.tree_flatten(tree, namespace='wrapping')
+    assert leaves == [3, 2, 1]
+    assert tree == optree.tree_unflatten(treespec, leaves)
+
+    assert hasattr(WrappingTestClass, 'tree_flatten')
+    assert hasattr(WrappingTestClass, 'tree_unflatten')
+    assert callable(WrappingTestClass.tree_flatten)
+    assert callable(WrappingTestClass.tree_unflatten)
