@@ -1,4 +1,4 @@
-# Copyright 2022-2024 MetaOPT Team. All Rights Reserved.
+# Copyright 2022-2025 MetaOPT Team. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,16 +13,18 @@
 # limitations under the License.
 # ==============================================================================
 
-# pylint: disable=missing-function-docstring,invalid-name,wrong-import-order
+# pylint: disable=missing-function-docstring,invalid-name
 
 import contextlib
 import itertools
+import os
 import pickle
 import platform
 import re
+import signal
 import subprocess
 import sys
-import sysconfig
+import tempfile
 import textwrap
 import weakref
 from collections import OrderedDict, UserList, defaultdict, deque
@@ -35,15 +37,81 @@ from helpers import (
     GLOBAL_NAMESPACE,
     NAMESPACED_TREE,
     PYPY,
+    TEST_ROOT,
     TREE_STRINGS,
     TREES,
     MyAnotherDict,
     MyDict,
+    Py_DEBUG,
+    disable_systrace,
     gc_collect,
     parametrize,
+    recursionlimit,
+    skipif_android,
+    skipif_ios,
+    skipif_pypy,
+    skipif_wasm,
 )
 
 
+@pytest.mark.skipif(
+    platform.machine().lower() not in ('x86_64', 'amd64'),
+    reason='Only run on x86_64 and AMD64 architectures',
+)
+@skipif_wasm
+@skipif_android
+@skipif_ios
+@skipif_pypy
+@disable_systrace
+def test_treespec_construct():
+    with pytest.raises(TypeError, match=re.escape('No constructor defined!')):
+        optree.PyTreeSpec()
+    treespec = optree.PyTreeSpec.__new__(optree.PyTreeSpec)
+    with pytest.raises(TypeError, match=re.escape('No constructor defined!')):
+        treespec.__init__()
+
+    env = {
+        key: value
+        for key, value in os.environ.items()
+        if not key.startswith(('PYTHON', 'PYTEST', 'COV_'))
+    }
+    script = textwrap.dedent(
+        r"""
+        import signal
+        import sys
+
+        import optree
+        import optree._C
+
+        for _ in range(32):
+            treespec = optree.PyTreeSpec.__new__(optree.PyTreeSpec)
+            try:
+                repr(treespec)
+            except optree._C.InternalError as ex:
+                assert 'src/treespec/serialization.cpp' in str(ex).replace('\\', '/')
+                sys.exit(0)
+        """,
+    ).strip()
+    returncode = 0
+    try:
+        with tempfile.TemporaryDirectory() as tmpdir:
+            subprocess.check_call(
+                [sys.executable, '-Walways', '-Werror', '-c', script],
+                cwd=tmpdir,
+                env=env,
+            )
+    except subprocess.CalledProcessError as ex:
+        returncode = abs(ex.returncode)
+        if 128 < returncode < 256:
+            returncode -= 128
+    assert returncode in (
+        0,
+        signal.SIGSEGV,
+        signal.SIGABRT,
+        0xC0000005,  # STATUS_ACCESS_VIOLATION on Windows
+    )
+
+
 def test_treespec_equal_hash():
     for i, tree1 in enumerate(TREES):
         treespec1 = optree.tree_structure(tree1)
@@ -188,6 +256,7 @@ def test_treespec_with_empty_dict_string_representation():
     assert str(optree.tree_structure({})) == r'PyTreeSpec({})'
 
 
+@disable_systrace
 def test_treespec_self_referential():
     class Holder:
         def __init__(self, value):
@@ -243,17 +312,19 @@ def test_treespec_self_referential():
     assert hash(other) == hash(other)
     assert hash(treespec) == hash(other)
 
-    if not PYPY:
-        with pytest.raises(RecursionError):
-            assert treespec != other
-
-    wr = weakref.ref(treespec)
-    del treespec, key, other
     gc_collect()
     if not PYPY:
+        with recursionlimit(64):
+            with pytest.raises(RecursionError):
+                assert treespec != other
+
+        wr = weakref.ref(treespec)
+        del treespec, key, other
+        gc_collect()
         assert wr() is None
 
 
+@disable_systrace
 def test_treeiter_self_referential():
     sentinel = object()
 
@@ -424,7 +495,7 @@ def test_treespec_with_namespace():
     dict_should_be_sorted=[False, True],
     dict_session_namespace=['', 'undefined', 'namespace'],
 )
-def test_treespec_pickle_round_trip(
+def test_treespec_pickle_roundtrip(
     tree,
     none_is_leaf,
     namespace,
@@ -456,12 +527,11 @@ class Foo:
         self.y = y
 
 
+@skipif_wasm
+@skipif_android
+@skipif_ios
 def test_treespec_pickle_missing_registration():
-    if (
-        sys.version_info[:2] == (3, 11)
-        and platform.system() == 'Windows'
-        and sysconfig.get_config_vars().get('EXT_SUFFIX', '').startswith('_d')
-    ):
+    if sys.version_info[:2] == (3, 11) and platform.system() == 'Windows' and Py_DEBUG:
         pytest.skip('Python 3.11 on Windows has a bug during PyStructSequence type deallocation.')
 
     optree.register_pytree_node(
@@ -474,6 +544,11 @@ def test_treespec_pickle_missing_registration():
     treespec = optree.tree_structure(Foo(0, 1), namespace='foo')
     serialized = pickle.dumps(treespec)
 
+    env = {
+        key: value
+        for key, value in os.environ.items()
+        if not key.startswith(('PYTHON', 'PYTEST', 'COV_'))
+    }
     try:
         output = subprocess.run(
             [
@@ -484,6 +559,8 @@ def test_treespec_pickle_missing_registration():
                     import pickle
                     import sys
 
+                    sys.path.insert(0, {str(TEST_ROOT)!r})
+
                     try:
                         treespec = pickle.loads({serialized!r})
                     except Exception as ex:
@@ -497,6 +574,9 @@ def test_treespec_pickle_missing_registration():
             capture_output=True,
             check=True,
             text=True,
+            encoding='utf-8',
+            cwd=TEST_ROOT,
+            env=env,
         )
         message = output.stdout.strip()
     except subprocess.CalledProcessError as ex:
@@ -1032,11 +1112,13 @@ def test_treespec_transform():
         match=re.escape('Expected the PyTreeSpec transform function returns a PyTreeSpec'),
     ):
         optree.treespec_transform(treespec, lambda _: None)
+
     with pytest.raises(
         TypeError,
         match=re.escape('Expected the PyTreeSpec transform function returns a PyTreeSpec'),
     ):
         optree.treespec_transform(treespec, None, lambda _: None)
+
     with pytest.raises(
         ValueError,
         match=(
@@ -1051,13 +1133,14 @@ def test_treespec_transform():
                 none_is_leaf=True,
             ),
         )
+
+    def fn(spec):
+        with optree.dict_insertion_ordered(True, namespace='undefined'):
+            return optree.treespec_dict(zip('abcd', spec.children()), namespace='undefined')
+
     with pytest.raises(ValueError, match=r'Expected treespec\(s\) with namespace .*, got .*\.'):
-
-        def fn(spec):
-            with optree.dict_insertion_ordered(True, namespace='undefined'):
-                return optree.treespec_dict(zip('abcd', spec.children()), namespace='undefined')
-
         optree.treespec_transform(namespaced_treespec, fn)
+
     with pytest.raises(
         ValueError,
         match=re.escape(
@@ -1066,6 +1149,7 @@ def test_treespec_transform():
         ),
     ):
         optree.treespec_transform(treespec, lambda _: optree.tree_structure([0, 1]))
+
     with pytest.raises(
         ValueError,
         match=re.escape(
@@ -1659,11 +1743,11 @@ def test_treespec_constructor(  # noqa: C901
 def test_treespec_constructor_namespace():
     @optree.register_pytree_node_class(namespace='mylist')
     class MyList(UserList):
-        def tree_flatten(self):
+        def __tree_flatten__(self):
             return self.data, None, None
 
         @classmethod
-        def tree_unflatten(cls, metadata, children):
+        def __tree_unflatten__(cls, metadata, children):
             return cls(children)
 
     with pytest.warns(
